@@ -1,21 +1,20 @@
 from functools import partial
-from collections import defaultdict, deque
+from collections import defaultdict
 from random import random
 
-from selenium.common.exceptions import NoSuchElementException
-from selenium.webdriver.common.action_chains import ActionChains
 from selenium.webdriver.common.by import By
 
-from .hv import HVDriver, searchxpath_fun
+from .hv import HVDriver
 from .hv_battle_ponychart import PonyChart
 from .hv_battle_item_provider import ItemProvider
 from .hv_battle_action_manager import ElementActionManager
 from .hv_battle_skill_manager import SkillManager
 from .hv_battle_buff_manager import BuffManager
 from .hv_battle_monster_status_manager import MonsterStatusManager
-from .hv_battle_log import LogProvider
-from .hv_battle_dashboard import BattleDashBoard
+
+# from .hv_battle_dashboard import BattleDashBoard
 from .pause_controller import PauseController
+from .hv_battle_observer_pattern import BattleDashboard
 
 MONSTER_DEBUFF_TO_CHARACTER_SKILL = {
     "Imperiled": "Imperil",
@@ -30,16 +29,6 @@ MONSTER_DEBUFF_TO_CHARACTER_SKILL = {
 }
 
 
-def return_false_on_nosuch(fun):
-    def wrapper(*args, **kwargs):
-        try:
-            return fun(*args, **kwargs)
-        except NoSuchElementException:
-            return False
-
-    return wrapper
-
-
 class StatThreshold:
     def __init__(
         self,
@@ -75,11 +64,10 @@ class BattleDriver(HVDriver):
     def __init__(self, *args, **kwargs) -> None:
         super().__init__(*args, **kwargs)
 
-        self.battle_dashboard = BattleDashBoard(self)
+        self.battle_dashboard = BattleDashboard(self)
         self.element_action_manager = ElementActionManager(self, self.battle_dashboard)
 
         self.with_ofc = "isekai" not in self.driver.current_url
-        self._logprovider = LogProvider(self, self.battle_dashboard)
         self._itemprovider = ItemProvider(self, self.battle_dashboard)
         self._skillmanager = SkillManager(self, self.battle_dashboard)
         self._buffmanager = BuffManager(self, self.battle_dashboard)
@@ -91,9 +79,9 @@ class BattleDriver(HVDriver):
 
     def clear_cache(self) -> None:
         # 重新解析戰鬥儀表板以獲取最新的怪物狀態
-        self.battle_dashboard.refresh()
         self._monsterstatusmanager.clear_cache()
-        self.round = self._logprovider.current_round
+        self.round = self.battle_dashboard.log_entries.current_round
+        self.battle_dashboard.update()
 
     def reset_pround(self) -> None:
         self.pround = self.round
@@ -112,23 +100,23 @@ class BattleDriver(HVDriver):
     def get_stat_percent(self, stat: str) -> float:
         match stat.lower():
             case "hp":
-                value = self.battle_dashboard.character.vitals.hp
+                value = self.battle_dashboard.character_vitals.hp
             case "mp":
-                value = self.battle_dashboard.character.vitals.mp
+                value = self.battle_dashboard.character_vitals.mp
             case "sp":
-                value = self.battle_dashboard.character.vitals.sp
+                value = self.battle_dashboard.character_vitals.sp
             case "overcharge":
-                value = self.battle_dashboard.character.vitals.overcharge
+                value = self.battle_dashboard.character_vitals.overcharge
             case _:
                 raise ValueError(f"Unknown stat: {stat}")
         return value
 
     @property
     def new_logs(self) -> list[str]:
-        new_logs = self._logprovider.get_new_logs()
+        new_logs = self.battle_dashboard.log_entries.current_lines
         # 固定寬度，假設最大 3 位數
         turn_str = f"Turn {self.turn:>5}"
-        round_str = f"Round {self._logprovider.current_round:>3} / {self._logprovider.total_round:<3}"
+        round_str = f"Round {self.battle_dashboard.log_entries.current_round:>3} / {self.battle_dashboard.log_entries.total_round:<3}"
         return [f"{turn_str} {round_str} {line}" for line in new_logs]
 
     def use_item(self, key: str) -> bool:
@@ -159,7 +147,6 @@ class BattleDriver(HVDriver):
     def system_monster_alive_ids(self) -> list[int]:
         return self._monsterstatusmanager.alive_system_monster_ids
 
-    @return_false_on_nosuch
     def check_hp(self) -> bool:
         if self.get_stat_percent("hp") < self.statthreshold.hp[0]:
             for fun in [
@@ -184,7 +171,6 @@ class BattleDriver(HVDriver):
 
         return False
 
-    @return_false_on_nosuch
     def check_mp(self) -> bool:
         if self.get_stat_percent("mp") < self.statthreshold.mp[0]:
             for fun in [
@@ -206,7 +192,6 @@ class BattleDriver(HVDriver):
 
         return False
 
-    @return_false_on_nosuch
     def check_sp(self) -> bool:
         if self.get_stat_percent("sp") < self.statthreshold.sp[0]:
             for fun in [
@@ -228,7 +213,6 @@ class BattleDriver(HVDriver):
 
         return False
 
-    @return_false_on_nosuch
     def check_overcharge(self) -> bool:
         if self._buffmanager.has_buff("Spirit Stance"):
             # If Spirit Stance is active, check if Overcharge and SP are below thresholds
@@ -251,16 +235,8 @@ class BattleDriver(HVDriver):
             return self.apply_buff("Spirit Stance")
         return False
 
-    @return_false_on_nosuch
     def go_next_floor(self) -> bool:
-        continue_images = [
-            "/y/battle/arenacontinue.png",
-            "/y/battle/grindfestcontinue.png",
-            "/y/battle/itemworldcontinue.png",
-        ]
-        continue_elements = self.driver.find_elements(
-            By.XPATH, searchxpath_fun(continue_images)
-        )
+        continue_elements = self.driver.find_elements(By.ID, "btcp")
 
         if continue_elements:
             self.element_action_manager.click_and_wait_log(continue_elements[0])
@@ -381,13 +357,15 @@ class BattleDriver(HVDriver):
             if self.get_stat_percent("overcharge") > 200 and self._buffmanager.has_buff(
                 "Spirit Stance"
             ):
-                monster_health = self.battle_dashboard.monster_list[n].vitals.health
-                if self._skillmanager.get_skill_status("Vital Strike"):
-                    self.attack_monster_by_skill(n, "Vital Strike")
-                elif monster_health < 25 and self._skillmanager.get_skill_status(
+                monster_health = self.battle_dashboard.monster_list.monsters[
+                    n
+                ].vitals.health
+                if monster_health < 25 and self._skillmanager.get_skill_status(
                     "Merciful Blow"
                 ):
                     self.attack_monster_by_skill(n, "Merciful Blow")
+                elif self._skillmanager.get_skill_status("Vital Strike"):
+                    self.attack_monster_by_skill(n, "Vital Strike")
             else:
                 self.attack_monster(n)
             self.last_debuff_monster_id["Imperiled"] = -1
@@ -405,19 +383,8 @@ class BattleDriver(HVDriver):
                 self.last_debuff_monster_id["Imperiled"] = n
         return True
 
-    def finish_battle(self) -> bool:
-        elements = self.driver.find_elements(
-            By.XPATH, searchxpath_fun(["/y/battle/finishbattle.png"])
-        )
-
-        if not elements:
-            return False
-
-        ActionChains(self.driver).move_to_element(elements[0]).click().perform()
-        return True
-
     def use_channeling(self) -> bool:
-        if "Channeling" in self.battle_dashboard.character.buffs:
+        if "Channeling" in self.battle_dashboard.character_buffs.buffs:
             skill_names = ["Regen", "Heartseeker"]
             skill2remaining: dict[str, float] = dict()
             for skill_name in skill_names:
@@ -440,9 +407,6 @@ class BattleDriver(HVDriver):
         # Print the current round logs
         print("\n".join(self.new_logs))
 
-        if self.finish_battle():
-            return "break"
-
         for fun in [
             self.go_next_floor,
             PonyChart(self).check,
@@ -450,13 +414,14 @@ class BattleDriver(HVDriver):
             self.check_mp,
             self.check_sp,
             self.check_overcharge,
-            partial(self.apply_buff, "Health Draught"),
-            partial(self.apply_buff, "Mana Draught"),
-            partial(self.apply_buff, "Spirit Draught"),
-            partial(self.apply_buff, "Regen"),
-            partial(self.apply_buff, "Scroll of Life"),
-            partial(self.apply_buff, "Absorb"),
-            partial(self.apply_buff, "Heartseeker"),
+            lambda: self.apply_buff("Health Draught"),
+            lambda: self.apply_buff("Mana Draught"),
+            lambda: self.apply_buff("Spirit Draught"),
+            lambda: self.apply_buff("Regen"),
+            lambda: self.apply_buff("Scroll of Life"),
+            lambda: self.apply_buff("Absorb"),
+            lambda: self.apply_buff("Scroll of Protection"),
+            lambda: self.apply_buff("Heartseeker"),
         ]:
             if fun():
                 return "continue"
